import os
from pathlib import Path
import argparse
import ee
import yaml
from tqdm import tqdm
from get_globfire import get_combined_fires, analyze_fires
from DataPreparation.DatasetPrepareService import DatasetPrepareService
from drive_downloader import DriveDownloader
from create_config import create_fire_config_globfire

config_data = {}

def get_full_geojson_path():
    return f"{config_data['geojson_dir']}combined_fires_{config_data['year']}.geojson"

def get_full_yaml_path():
    ROOT = Path(__file__).resolve().parent
    config_dir = ROOT / "config" / f"us_fire_{config_data['year']}_1e7.yml"
    return config_dir

def sync_drive_path_with_year():
    drive_path = f"EarthEngine_WildfireSpreadTS_{config_data['year']}"
    config_data['drive_dir'] = drive_path

def load_yaml_config(path):
    if os.path.exists(path):
        with open(path, 'r') as f:
            return yaml.safe_load(f) or {}
    return {}

def save_yaml_config(config, path):
    os.makedirs(os.path.dirname(path), exist_ok=True)
    with open(path, 'w') as f:
        yaml.dump(config, f, sort_keys=False)

def load_fire_config(yaml_path):
    with open(
        yaml_path, "r", encoding="utf8"
    ) as f:
        config = yaml.load(f, Loader=yaml.FullLoader)
    return config


# from get_globfire.py
def generate_geojson():
    # Get both daily and final perimeters
    combined_gdf, daily_gdf, final_gdf = get_combined_fires(
        config_data['year'], config_data['min_size'] 
    )

    if combined_gdf is not None:
        print(f"\nAnalysis Results for {config_data['year']}:")

        print("\nCombined Perimeters:")
        combined_stats = analyze_fires(combined_gdf)
        for key, value in combined_stats.items():
            print(f"{key}: {value}")

        if daily_gdf is not None:
            print("\nDaily Perimeters:")
            daily_stats = analyze_fires(daily_gdf)
            for key, value in daily_stats.items():
                print(f"{key}: {value}")

        if final_gdf is not None:
            print("\nFinal Perimeters:")
            final_stats = analyze_fires(final_gdf)
            for key, value in final_stats.items():
                print(f"{key}: {value}")

        # Temporal distribution
        print("\nFires by month:")
        monthly_counts = (
            combined_gdf.groupby([combined_gdf["date"].dt.month, "source"])
            .size()
            .unstack(fill_value=0)
        )
        print(monthly_counts)

    # drop everything that does not have at least 2 Id in combined_gdf
    combined_gdf_reduced = combined_gdf[
        combined_gdf["Id"].isin(
            combined_gdf["Id"]
        .value_counts()[combined_gdf["Id"].value_counts() > 1]
        .index
        )
    ]  # save to geojson

    geojson_path = get_full_geojson_path()
    combined_gdf_reduced.to_file(
        geojson_path,
        driver="GeoJSON",
    )


# from DatasetPrepareService.py
def export_data(yaml_path):
    #project_id = config_data['project_id']
    
    # fp = FirePred()
    config = load_fire_config(yaml_path)
    # print(f"[LOG] from export_data, yaml path: {yaml_path}")
    # print(f"[LOG] from export_data, config: {config}")
    fire_names = list(config.keys())
    for non_fire_key in ["output_bucket", "rectangular_size", "year"]:
        fire_names.remove(non_fire_key)
    locations = fire_names

    # Track any failures
    failed_locations = []

    # Process each location
    for location in tqdm(locations):
        print(f"\nFailed locations so far: {failed_locations}")
        print(f"Current Location: {location}")

        dataset_pre = DatasetPrepareService(location=location, config=config)

        try:
            print("Trying to export to Google Drive")
            dataset_pre.extract_dataset_from_gee_to_drive("32610", n_buffer_days=4)
        except Exception as e:
            print(f"Failed on {location}: {str(e)}")
            failed_locations.append(location)
            continue

    if failed_locations:
        print("\nFailed locations:")
        for loc in failed_locations:
            print(f"- {loc}")
    else:
        print("\nAll locations processed successfully!")


def main():
    global config_data
    base_parser = argparse.ArgumentParser(add_help=False)
    base_parser.add_argument('--config', 
                             type=str,default="./config_options.yml" ,
                             help="Path to JSON config file")
    args_partial, _ = base_parser.parse_known_args()

    # Load from YAML config (if given)
    config_path = args_partial.config
    config_data = load_yaml_config(config_path) if config_path else {}

    # Full parser
    parser = argparse.ArgumentParser(
        parents=[base_parser],
        description="Generate fire config YAML from GeoJSON."
    )
    parser.add_argument(
        "--year", type=str, help="Year of fire parameters."
    )

    parser.add_argument("--min-size", type=float, 
                        # default=configuration.MIN_SIZE,
                        )

    parser.add_argument(
        "--output",
        type=str,
        # default=configuration.OUTPUT_DIR,
        help="local directory where the TIFF files will go.",
    )

    parser.add_argument(
        "--drive-dir",
        type=str,
        # default=configuration.DATA_DIR,
        help="The google drive directory for TIFF files",
    )

    parser.add_argument(
        "--credentials",
        type=str,
        # default=configuration.CREDENTIALS,
        help="Path to Google OAuth credentials JSON.",
    )

    parser.add_argument(
        "--project-id",
        type=str,
        # default=configuration.PROJECT,
        help="Project ID for Google Cloud",
    )

    parser.add_argument(
        "--geojson-dir",
        type=str,
        # default=configuration.DATA_DIR,
        help="Directory to store geojson files",
    )

    parser.add_argument(
        "--download",
        # type=bool,
        action="store_true",
        help="Download TIFF files from google drive.",
    )
    parser.add_argument("--export-data",
                        action="store_true",
                        help="Export to Google Drive")

    parser.add_argument("--show-config",
                        action="store_true",
                        help="Show current configuration.")

    parser.add_argument("--force-new-geojson",
                        action="store_true",
                        help="Force generate new geojson.")

    parser.add_argument("--sync-year",
                        action="store_true",
                        help="Syncs the year to the input/output files")

    args = parser.parse_args()

    # Update config_data with any non-None CLI args (override)
    for key in ["year","min_size","output","drive_dir",
                "credentials","project_id","geojson_dir",
                "download", "export_data", "show_config",
                "force_new_geojson", "sync_year"]:
        val = getattr(args,key)
        if val is not None:
            config_data[key]=val


    # save dictionary back to yaml file
    if config_path:
        save_yaml_config(config_data, config_path)

    # use or generate GeoJSON
    ee.Authenticate()
    ee.Initialize(project=config_data['project_id'])


    if(config_data['sync_year']):
        sync_drive_path_with_year()


    geojson_path = get_full_geojson_path()
    if (not os.path.exists(geojson_path) or config_data['force_new_geojson']):
        print("Generating Geojson...")
        generate_geojson()

    # generate the YAML output config
    yaml_path = get_full_yaml_path()
    # print(f"[LOG] Yaml path from main(): {yaml_path}")
    create_fire_config_globfire(geojson_path, yaml_path, config_data['year'])
    
    if(config_data['show_config']):
        print(config_data)

    if(config_data['export_data']):
        print("Exporting data...")
        export_data(yaml_path)   

    if(config_data['download']):
        downloader = DriveDownloader(config_data['credentials'])
        downloader.download_folder(config_data['drive_dir'], config_data['output'])






if __name__ == "__main__":
    main()



import os
from google.oauth2.credentials import Credentials
from google_auth_oauthlib.flow import InstalledAppFlow
from google.auth.transport.requests import Request
from googleapiclient.discovery import build
from googleapiclient.http import MediaIoBaseDownload
import io
from pathlib import Path
from tqdm import tqdm

SCOPES = ['https://www.googleapis.com/auth/drive.readonly']

class DriveDownloader:
    def __init__(self, credentials):
        self.creds = credentials
        self.service = self._get_drive_service()
        
    def _get_drive_service(self):
        creds = None
        if os.path.exists('token.json'):
            creds = Credentials.from_authorized_user_file('token.json', SCOPES)
        
        if not creds or not creds.valid:
            if creds and creds.expired and creds.refresh_token:
                creds.refresh(Request())
            else:
                flow = InstalledAppFlow.from_client_secrets_file(
                    self.creds, SCOPES)
                creds = flow.run_local_server(port=0)
            
            with open('token.json', 'w') as token:
                token.write(creds.to_json())
        
        return build('drive', 'v3', credentials=creds)

    def _get_folder_id(self, folder_path: str):
        """Get folder ID by traversing the path."""
        parts = folder_path.strip('/').split('/')
        parent_id = 'root'
        
        for part in parts:
            query = f"name='{part}' and '{parent_id}' in parents and mimeType='application/vnd.google-apps.folder'"
            results = self.service.files().list(q=query, spaces='drive').execute()
            items = results.get('files', [])
            
            if not items:
                raise ValueError(f"Folder '{part}' not found in path '{folder_path}'")
            parent_id = items[0]['id']
            
        return parent_id

    def download_folder(self, drive_folder_path: str, local_path: str):
        """Download all files from the specified Drive folder."""
        try:
            folder_id = self._get_folder_id(drive_folder_path)
            output_dir = Path(local_path)
            output_dir.mkdir(parents=True, exist_ok=True)
            
            # Get all files in the folder with pagination
            files = []
            page_token = None
            while True:
                results = self.service.files().list(
                    q=f"'{folder_id}' in parents",
                    spaces='drive',
                    pageSize=1000,
                    fields='nextPageToken, files(id, name)',
                    pageToken=page_token
                ).execute()
                files.extend(results.get('files', []))
                page_token = results.get('nextPageToken')
                if not page_token:
                    break
            
            print(f"Found {len(files)} files")
            
            for file in tqdm(files, desc="Downloading files"):
                request = self.service.files().get_media(fileId=file['id'])
                fh = io.BytesIO()
                downloader = MediaIoBaseDownload(fh, request)
                
                done = False
                while not done:
                    _, done = downloader.next_chunk()
                
                output_path = output_dir / file['name']
                with open(output_path, 'wb') as f:
                    f.write(fh.getvalue())
                    
        except Exception as e:
            print(f"Error downloading folder: {str(e)}")
            raise

def main():
    # drive_folder = input("Enter Drive folder path (e.g., EarthEngine_WildfireSpreadTS): ")
    # local_folder = input("Enter local download path: ")
    
    downloader = DriveDownloader()
    # downloader.download_folder(drive_folder, local_folder)
    # downloader.download_folder(configuration.DRIVE_DIR, configuration.OUTPUT_DIR)
    print("\nDownload completed!")

if __name__ == '__main__':
    main()



import datetime

import math
from ee import Image #type: ignore
from ee import ImageCollection #type: ignore
from ee import FeatureCollection #type: ignore
from ee import Filter #type: ignore
from ee import Reducer #type: ignore
from ee import Number #type: ignore
from ee import String #type: ignore
from ee import Terrain #type: ignore
from ee import Geometry #type: ignore


class FirePred:
    def __init__(self):
        """_summary_ This class describes which data to extract how from Google Earth Engine. 
        The init defines the different source data products to use. 
        """
        self.name = "FirePred"
        # Digital elevation model
        self.srtm = Image("USGS/SRTMGL1_003")
        # Land cover
        self.landcover = ImageCollection("MODIS/061/MCD12Q1")
        # Weather data (GRIDMET)
        self.weather = ImageCollection("IDAHO_EPSCOR/GRIDMET")
        # Weather forecast data (GFS)
        self.weather_forecast = ImageCollection('NOAA/GFS0P25')
        # Drought data (GRIDMET)
        self.drought = ImageCollection("GRIDMET/DROUGHT")
        # VIIRS surface reflectance, needed updating to current dataset
        self.viirs = ImageCollection('NASA/VIIRS/002/VNP09GA')
        # VIIRS active fire product
        self.viirs_af = FeatureCollection('projects/grand-drive-285514/assets/afall')
        # self.viirs_af = ee.FeatureCollection('projects/ee-earthdata/assets/viirs_J1V_af_points') # if this doesnt work try the SUOMI points
        # self.viirs_af = ee.FeatureCollection('projects/ee-earthdata/assets/fire_archive_SV-C2_574505') # This is the historical archive (02-01-2012 to end of 2024)
        # VIIRS vegetation index, needed updating to new dataset
        self.viirs_veg_idx = ImageCollection("NASA/VIIRS/002/VNP13A1")

    def compute_daily_features(self, start_time:str, end_time:str, geometry:Geometry):
        """_summary_ Compute the daily features in Google Earth Engine.

        Args:
            start_time (str): _description_
            end_time (str): _description_
            geometry (ee.Geometry): _description_

        Returns:
            ee.ImageCollection: _description_ ImageCollection containing one image, 
            with all desired features for the given day, inside the given geometry.
        """


        # Time objects we need later. We add "000" to timestamps, because GEE has timestamps with miliseconds,
        # but datetime doesn't by default
        today_string = start_time[:-6].replace("-", "")
        today = datetime.datetime.strptime(start_time[:-6], '%Y-%m-%d')
        today_timestamp = int(datetime.datetime.timestamp(today)) * 1000

        today = datetime.datetime.strptime(start_time[:-6], '%Y-%m-%d')
        tomorrow = today + datetime.timedelta(days=1)
        today_string = start_time[:-6].replace("-", "")
        
        # Convert to Unix millisecond timestamps for VIIRS AF filtering
        start_timestamp = int(today.timestamp()) * 1000
        end_timestamp = int(tomorrow.timestamp()) * 1000
        today_timestamp = start_timestamp  # For drought data

        # Weather Data
        # Median is used to turn ee.ImageCollection into a single ee.Image.
        # Each ImageCollection should only contain a single image at this point.
        weather = self.weather.filterDate(start_time, end_time).filterBounds(geometry)
        precipitation = weather.select('pr').median().rename("total precipitation")
        wind_direction = weather.select('th').median().rename("wind direction")
        temperature_min = weather.select('tmmn').median().rename("minimum temperature")
        temperature_max = weather.select('tmmx').median().rename("maximum temperature")
        energy_release_component = weather.select('erc').median().rename("energy release component")
        specific_humidity = weather.select('sph').median().rename("specific humidity")
        wind_velocity = weather.select('vs').median().rename("wind speed")

        # Take forecasts made at midnight (00), and that tell us something about the hours between 01 and 24.
        # Important: The forecasts at 00 contain six features instead of nine, like all others.
        weather_forecast = self.weather_forecast.filter(
            Filter.gte("system:index", today_string + "00F01")).filter(
            Filter.lte("system:index", today_string + "00F24")
        ).filterBounds(geometry)
        forecast_temperature = weather_forecast.select("temperature_2m_above_ground").mean().rename(
            "forecast temperature")
        forecast_specific_humidity = weather_forecast.select("specific_humidity_2m_above_ground").mean().rename(
            "forecast specific humidity")
        forecast_u_wind = weather_forecast.select("u_component_of_wind_10m_above_ground").mean()
        forecast_v_wind = weather_forecast.select("v_component_of_wind_10m_above_ground").mean()

        # Transform from u/v to direction and speed, to align with GRIDMET and DEM data
        forecast_wind_speed = forecast_u_wind.multiply(forecast_u_wind).add(
            forecast_v_wind.multiply(forecast_v_wind)).sqrt().rename("forecast wind speed")
        forecast_wind_direction = forecast_v_wind.divide(forecast_u_wind).atan()
        forecast_wind_direction = forecast_wind_direction.divide(2 * math.pi).multiply(360).rename(
            "forecast wind direction")

        # Rain forecasts were changed: From rain within the one-hour interval to cumulative rain during the day so far
        forecast_rain_change_date = datetime.datetime.strptime("2019-11-07T06:00:00", '%Y-%m-%dT%H:%M:%S')
        forecast_rain = weather_forecast.select("total_precipitation_surface")
        if today <= forecast_rain_change_date:
            forecast_rain = forecast_rain.reduce(Reducer.sum())
        else:
            forecast_rain = forecast_rain.reduce(Reducer.last())
        forecast_rain.rename("forecast total precipitation")
        # Elevation Data
        elevation = self.srtm.select('elevation')
        slope = Terrain.slope(elevation)
        aspect = Terrain.aspect(elevation)

        # Drought Data
        # Only available every fifth day, but we can find the valid entry via time_start and time_end
        drought_index = self.drought \
            .filter(Filter.lte("system:time_start", today_timestamp)) \
            .filter(Filter.gte("system:time_end", today_timestamp)) \
            .select('pdsi').median()
        igbp_land_cover = self.landcover.filterDate(start_time[:4] + '-01-01', start_time[:4] + '-12-31').filterBounds(
            geometry).select('LC_Type1').median()

        # Turn acq_time (String) into acq_hour (int)
        def add_acq_hour(feature):
            acq_time_str = String(feature.get("ACQ_TIME"))
            acq_time_int = Number.parse(acq_time_str)
            return feature.set({"acq_hour": acq_time_int})

        # VIIRS IMG and AF product
        viirs_img = self.viirs.filterDate(start_time, end_time).filterBounds(geometry).select(
            ['M11', 'I2', 'I1']).median()
        viirs_veg_idc = self.viirs_veg_idx.filterDate((
                datetime.datetime.strptime(end_time[:-6], '%Y-%m-%d') + datetime.timedelta(-15)).strftime(
            '%Y-%m-%d'), end_time).filterBounds(geometry).select(['NDVI', 'EVI2']).reduce(
            Reducer.last())

        # Updated VIIRS AF filtering to use Unix millisecond timestamps
        viirs_af_img = self.viirs_af.map(add_acq_hour).filterBounds(geometry) \
            .filter(Filter.gte('ACQ_DATE', start_timestamp)) \
            .filter(Filter.lt('ACQ_DATE', end_timestamp)) \
            .filter(Filter.neq('CONFIDENCE', 'l')).map(self.get_buffer) \
            .reduceToImage(['acq_hour'], Reducer.last()) \
            .rename(['active fire'])

        return ImageCollection(Image(
            [viirs_img, viirs_veg_idc, precipitation, wind_velocity, wind_direction, temperature_min, temperature_max,
             energy_release_component, specific_humidity, slope, aspect,
             elevation, drought_index, igbp_land_cover,
             forecast_rain, forecast_wind_speed, forecast_wind_direction, forecast_temperature,
             forecast_specific_humidity,
             viirs_af_img]))

    def get_buffer(self, feature):
        return feature.buffer(375 / 2).bounds()
